@@ -9,6 +9,7 @@ from matplotlib.style import available
 import numpy as np
 import sys
 import warnings
+from copy import deepcopy
 
 from argparse import ArgumentParser
 import yaml
@@ -41,7 +42,7 @@ class AutoPipeline(Pipeline):
     """
 
     def __init__(self,
-                 input_directory,
+                 input_directory="",
                  output_directory="",
                  modalities="CT",
                  spacing=(1., 1., 0.),
@@ -51,17 +52,21 @@ class AutoPipeline(Pipeline):
                  show_progress=False,
                  warn_on_error=False,
                  overwrite=False,
-                 is_nnunet=False,
+                 nnunet=False,
                  train_size=1.0,
                  random_state=42,
                  read_yaml_label_names=False,
                  ignore_missing_regex=False,
                  roi_yaml_path="",
                  custom_train_test_split=False,
-                 is_nnunet_inference=False,
+                 nnunet_inference=False,
                  dataset_json_path="",
                  continue_processing=False,
-                 dry_run=False):
+                 dry_run=False,
+                 verbose=False,
+                 update=False,
+                 roi_select_first=False,
+                 roi_separate=False):
         """Initialize the pipeline.
 
         Parameters
@@ -86,7 +91,7 @@ class AutoPipeline(Pipeline):
             Whether to warn on errors
         overwrite: bool, default=False
             Whether to write output files even if existing output files exist
-        is_nnunet: bool, default=False
+        nnunet: bool, default=False
             Whether to format the output for nnunet
         train_size: float, default=1.0
             Proportion of the dataset to use for training, as a decimal
@@ -100,7 +105,7 @@ class AutoPipeline(Pipeline):
             The path to the yaml file defining regexes
         custom_train_test_split: bool, default=False
             Whether to use a custom train/test split. The remaining patients will be randomly split using train_size and random_state
-        is_nnunet_inference: bool, default=False
+        nnunet_inference: bool, default=False
             Whether to format the output for nnUNet inference
         dataset_json_path: str, default=""
             The path to the dataset.json file for nnUNet inference
@@ -114,40 +119,52 @@ class AutoPipeline(Pipeline):
 
         self.continue_processing = continue_processing
         self.dry_run = dry_run
+        self.v = verbose
 
         if dry_run:
-            is_nnunet = False
-            is_nnunet_inference = False
+            nnunet = False
+            nnunet_inference = False
 
         if dry_run and continue_processing:
             raise ValueError("Cannot continue processing a dry run. Set --continue_processing to False to do a dry run.")
 
         if not dry_run and output_directory == "":
             raise ValueError("Must specify an output directory")
-        # pipeline configuration
+        
+        # input/output directory configuration
         if not os.path.isabs(input_directory):
             input_directory = pathlib.Path(os.getcwd(), input_directory).as_posix()
+        else:
+            input_directory = pathlib.Path(input_directory).as_posix()  # consistent parsing. ensures last child directory doesn't end with slash
+        
         if not os.path.isabs(output_directory):
             output_directory = pathlib.Path(os.getcwd(), output_directory).as_posix()
+        else:
+            output_directory = pathlib.Path(output_directory).as_posix() # consistent parsing. ensures last child directory doesn't end with slash
+
+        # check/make output directory
         if not os.path.exists(output_directory):
-            # raise FileNotFoundError(f"Output directory {output_directory} does not exist")
             os.makedirs(output_directory)
+
+        # check input directory exists
         if not os.path.exists(input_directory):
             raise FileNotFoundError(f"Input directory {input_directory} does not exist")
+        
         self.input_directory = pathlib.Path(input_directory).as_posix()
         self.output_directory = pathlib.Path(output_directory).as_posix()
         
-        if not is_nnunet and continue_processing and not os.path.exists(pathlib.Path(output_directory, ".temp").as_posix()):
+        # if wanting to continue processing but no .temp folders
+        if not nnunet and continue_processing and not os.path.exists(pathlib.Path(output_directory, ".temp").as_posix()):
             raise FileNotFoundError(f"Cannot continue processing. .temp directory does not exist in {output_directory}. Run without --continue_processing to start from scratch.")
 
         study_name = os.path.split(self.input_directory)[1]
-        if is_nnunet_inference:
+        if nnunet_inference:
             roi_yaml_path = ""
             custom_train_test_split = False
-            is_nnunet = False
-            if modalities != "CT" or modalities != "MR":
+            nnunet = False
+            if modalities != "CT" and modalities != "MR":
                 raise ValueError("nnUNet inference can only be run on image files. Please set modalities to 'CT' or 'MR'")
-        if is_nnunet:
+        if nnunet:
             self.base_output_directory = self.output_directory
             if not os.path.exists(pathlib.Path(self.output_directory, "nnUNet_preprocessed").as_posix()):
                 os.makedirs(pathlib.Path(self.output_directory, "nnUNet_preprocessed").as_posix())
@@ -190,28 +207,7 @@ class AutoPipeline(Pipeline):
 
             #continue processing operations
             self.finished_subjects = [pathlib.Path(e).name[:-4] for e in glob.glob(pathlib.Path(self.output_directory, ".temp", "*.pkl").as_posix())] #remove the .pkl
-            if continue_processing:
-                with open(pathlib.Path(self.output_directory, ".temp", "init_parameters.pkl").as_posix(), "rb") as f:
-                    parameters = dill.load(f)
-                    input_directory = parameters["input_directory"]
-                    output_directory = parameters["output_directory"]
-                    modalities = parameters["modalities"]
-                    spacing = parameters["spacing"]
-                    n_jobs = parameters["n_jobs"]
-                    visualize = parameters["visualize"]
-                    missing_strategy = parameters["missing_strategy"]
-                    show_progress = parameters["show_progress"]
-                    warn_on_error = parameters["warn_on_error"]
-                    overwrite = parameters["overwrite"]
-                    is_nnunet = parameters["is_nnunet"]
-                    train_size = parameters["train_size"]
-                    random_state = parameters["random_state"]
-                    read_yaml_label_names = parameters["read_yaml_label_names"]
-                    ignore_missing_regex = parameters["ignore_missing_regex"]
-                    roi_yaml_path = parameters["roi_yaml_path"]
-                    custom_train_test_split = parameters["custom_train_test_split"]
-                    is_nnunet_inference = parameters["is_nnunet_inference"]
-                    dataset_json_path = parameters["dataset_json_path"]
+            
 
         super().__init__(
             n_jobs=n_jobs,
@@ -221,8 +217,8 @@ class AutoPipeline(Pipeline):
         self.overwrite = overwrite
         self.spacing = spacing
         self.existing = [None] #self.existing_patients()
-        self.is_nnunet = is_nnunet
-        if is_nnunet or is_nnunet_inference:
+        self.is_nnunet = nnunet
+        if nnunet or nnunet_inference:
             self.nnunet_info = {}
         else:
             self.nnunet_info = None
@@ -231,7 +227,9 @@ class AutoPipeline(Pipeline):
         self.label_names = {}
         self.ignore_missing_regex = ignore_missing_regex
         self.custom_train_test_split = custom_train_test_split
-        self.is_nnunet_inference = is_nnunet_inference
+        self.is_nnunet_inference = nnunet_inference
+        self.roi_select_first = roi_select_first
+        self.roi_separate = roi_separate
 
         if roi_yaml_path != "" and not read_yaml_label_names:
             warnings.warn("The YAML will not be read since it has not been specified to read them. To use the file, run the CLI with --read_yaml_label_names")
@@ -260,26 +258,26 @@ class AutoPipeline(Pipeline):
             elif not isinstance(k, str):
                 raise ValueError(f"Label names must be a string. Got {k} for {v}")
 
-        if self.train_size == 1.0 and is_nnunet:
+        if self.train_size == 1.0 and nnunet:
             warnings.warn("Train size is 1, all data will be used for training")
         
-        if self.train_size == 0.0 and is_nnunet:
+        if self.train_size == 0.0 and nnunet:
             warnings.warn("Train size is 0, all data will be used for testing")
 
-        if self.train_size != 1 and not self.is_nnunet:
+        if self.train_size != 1 and not nnunet:
             warnings.warn("Cannot run train/test split without nnunet, ignoring train_size")
 
-        if self.train_size > 1 or self.train_size < 0 and self.is_nnunet:
+        if self.train_size > 1 or self.train_size < 0 and nnunet:
             raise ValueError("train_size must be between 0 and 1")
         
-        if is_nnunet and (not read_yaml_label_names or self.label_names == {}):
+        if nnunet and (not read_yaml_label_names or self.label_names == {}):
             raise ValueError("YAML label names must be provided for nnunet")
         
-        if custom_train_test_split and not is_nnunet:
+        if custom_train_test_split and not nnunet:
             raise ValueError("Cannot use custom train/test split without nnunet")
 
         custom_train_test_split_path = pathlib.Path(self.input_directory, "custom_train_test_split.yaml").as_posix()
-        if custom_train_test_split and is_nnunet:
+        if custom_train_test_split and nnunet:
             if os.path.exists(custom_train_test_split_path):
                 with open(custom_train_test_split_path, "r") as f:
                     try:
@@ -313,32 +311,32 @@ class AutoPipeline(Pipeline):
         if self.is_nnunet:
             self.nnunet_info["modalities"] = {"CT": "0000"} #modality to 4-digit code
 
-        if is_nnunet_inference:
+        if nnunet_inference:
             if not os.path.exists(dataset_json_path):
                 raise FileNotFoundError(f"No file named {dataset_json_path} found. Image modality definitions are required for nnUNet inference")
             else:
                 with open(dataset_json_path, "r") as f:
                     self.nnunet_info["modalities"] = {v: k.zfill(4) for k, v in json.load(f)["modality"].items()}
 
-        #input operations
-        self.input = ImageAutoInput(input_directory, modalities, n_jobs, visualize)
-        
+        # Input operations
+        self.input = ImageAutoInput(input_directory, modalities, n_jobs, visualize, update)
         self.output_df_path = pathlib.Path(self.output_directory, "dataset.csv").as_posix()
-        #Output component table
+
+        # Output component table
         self.output_df = self.input.df_combined
-        #Name of the important columns which needs to be saved    
+
+        # Name of the important columns which needs to be saved
         self.output_streams = self.input.output_streams
         
         # image processing ops
         self.resample = Resample(spacing=self.spacing)
-        self.make_binary_mask = StructureSetToSegmentation(roi_names=self.label_names, continuous=False) # "GTV-.*"
+        self.make_binary_mask = StructureSetToSegmentation(roi_names=self.label_names, continuous=False)
 
         # output ops
         self.output = ImageAutoOutput(self.output_directory, self.output_streams, self.nnunet_info, self.is_nnunet_inference)
         
         self.existing_roi_names = {"background": 0}
-        # self.existing_roi_names.update({k:i+1 for i, k in enumerate(self.label_names.keys())})
-        if is_nnunet or is_nnunet_inference:
+        if nnunet or nnunet_inference:
             self.total_modality_counter = {}
             self.patients_with_missing_labels = set()
         
@@ -399,11 +397,14 @@ class AutoPipeline(Pipeline):
                 mult_conn = colname.split("_")[-1].isnumeric()
                 num = colname.split("_")[-1]
 
-                # print(output_stream) #could include for verbose
+                if self.v:
+                    print("output_stream:", output_stream)
 
                 if read_results[i] is None:
                     print("The subject id: {} has no {}".format(subject_id, colname))
                     pass
+
+                # Process image (CT/MR)
                 elif modality == "CT" or modality == 'MR':
                     image = read_results[i].image
                     if len(image.GetSize()) == 4:
@@ -413,7 +414,8 @@ class AutoPipeline(Pipeline):
                         extractor.SetIndex([0, 0, 0, 0])    
                         
                         image = extractor.Execute(image)
-                        # print(image.GetSize()) #could include with verbose
+                        if self.v:
+                            print("image.GetSize():", image.GetSize())
                     try:
                         image = self.resample(image)
                     except Exception as e:
@@ -456,6 +458,8 @@ class AutoPipeline(Pipeline):
 
 
                     print(subject_id, " SAVED IMAGE")
+
+                # Process dose
                 elif modality == "RTDOSE":
                     try: #For cases with no image present
                         doses = read_results[i].resample_dose(image)
@@ -464,10 +468,7 @@ class AutoPipeline(Pipeline):
                         doses = read_results[i]
                     
                     # save output
-                    if not mult_conn:
-                        self.output(subject_id, doses, output_stream)
-                    else:
-                        self.output(f"{subject_id}_{num}", doses, output_stream)
+                    self.output(subject_id, doses, output_stream)
                     metadata[f"size_{output_stream}"] = str(doses.GetSize())
                     metadata[f"metadata_{colname}"] = [read_results[i].get_metadata()]
 
@@ -475,17 +476,27 @@ class AutoPipeline(Pipeline):
                         metadata.update(doses.metadata)
 
                     print(subject_id, " SAVED DOSE")
+                
+                # Process contour
                 elif modality == "RTSTRUCT":
                     num_rtstructs += 1
-                    #For RTSTRUCT, you need image or PT
+                    # For RTSTRUCT, you need image or PT
                     structure_set = read_results[i]
                     conn_to = output_stream.split("_")[-1]
 
                     # make_binary_mask relative to ct/pet
-                    if conn_to == "CT" or conn_to == "MR":
-                        mask = self.make_binary_mask(structure_set, image, self.existing_roi_names, self.ignore_missing_regex)
-                    elif conn_to == "PT":
-                        mask = self.make_binary_mask(structure_set, pet, self.existing_roi_names, self.ignore_missing_regex)
+                    if conn_to in ["CT", "MR", "PT"]:
+                        if conn_to == "CT" or conn_to == "MR":
+                            img = image
+                        elif conn_to == "PT":
+                            img = pet
+                        
+                        mask = self.make_binary_mask(structure_set, img, 
+                                                     self.existing_roi_names, 
+                                                     self.ignore_missing_regex, 
+                                                     roi_select_first=self.roi_select_first, 
+                                                     roi_separate=self.roi_separate)
+
                     else:
                         raise ValueError("You need to pass a reference CT or PT/PET image to map contours to.")
                     
@@ -515,10 +526,10 @@ class AutoPipeline(Pipeline):
                         if name not in self.existing_roi_names.keys():
                             self.existing_roi_names[name] = len(self.existing_roi_names)
                     mask.existing_roi_names = self.existing_roi_names
-                    # print(self.existing_roi_names,"alskdfj")
 
-                    # save output
-                    # print(mask.GetSize()) #could include with verbose
+                    
+                    if self.v:
+                        print("mask.GetSize():", mask.GetSize())
                     mask_arr = np.transpose(sitk.GetArrayFromImage(mask))
                     
                     if self.is_nnunet:
@@ -530,22 +541,23 @@ class AutoPipeline(Pipeline):
                         else:
                             self.output(subject_id, sparse_mask, output_stream, nnunet_info=self.nnunet_info, label_or_image="labels", train_or_test="Ts")
                     else:
+                    
                     # if there is only one ROI, sitk.GetArrayFromImage() will return a 3d array instead of a 4d array with one slice
                         if len(mask_arr.shape) == 3:
                             mask_arr = mask_arr.reshape(1, mask_arr.shape[0], mask_arr.shape[1], mask_arr.shape[2])
                         
-                        # print(mask_arr.shape) #could include with verbose
+                        if self.v:
+                            print(mask_arr.shape)
+
                         roi_names_list = list(mask.roi_names.keys())
                         for i in range(mask_arr.shape[0]):
                             new_mask = sitk.GetImageFromArray(np.transpose(mask_arr[i]))
                             new_mask.CopyInformation(mask)
                             new_mask = Segmentation(new_mask)
                             mask_to_process = new_mask
-                            if not mult_conn:
-                                # self.output(roi_names_list[i], mask_to_process, output_stream)
-                                self.output(subject_id, mask_to_process, output_stream, True, roi_names_list[i])
-                            else:
-                                self.output(f"{subject_id}_{num}", mask_to_process, output_stream, True, roi_names_list[i])
+                            
+                            # output
+                            self.output(subject_id, mask_to_process, output_stream, True, roi_names_list[i])
                     
                     if hasattr(structure_set, "metadata") and structure_set.metadata is not None:
                         metadata.update(structure_set.metadata)
@@ -553,6 +565,8 @@ class AutoPipeline(Pipeline):
                     metadata[f"metadata_{colname}"] = [structure_set.roi_names]
 
                     print(subject_id, "SAVED MASK ON", conn_to)
+                
+                # Process PET
                 elif modality == "PT":
                     try:
                         #For cases with no image present
@@ -561,10 +575,8 @@ class AutoPipeline(Pipeline):
                         Warning("No CT image present. Returning PT/PET image without resampling.")
                         pet = read_results[i]
 
-                    if not mult_conn:
-                        self.output(subject_id, pet, output_stream)
-                    else:
-                        self.output(f"{subject_id}_{num}", pet, output_stream)
+                    # output
+                    self.output(subject_id, pet, output_stream)
                     metadata[f"size_{output_stream}"] = str(pet.GetSize())
                     metadata[f"metadata_{colname}"] = [read_results[i].get_metadata()]
 
@@ -574,6 +586,7 @@ class AutoPipeline(Pipeline):
                     print(subject_id, " SAVED PET")
                 
                 metadata[f"output_folder_{colname}"] = pathlib.Path(subject_id, colname).as_posix()
+            
             #Saving all the metadata in multiple text files
             metadata["Modalities"] = str(list(subject_modalities))
             metadata["numRTSTRUCTs"] = num_rtstructs
@@ -584,6 +597,9 @@ class AutoPipeline(Pipeline):
             return 
     
     def save_data(self):
+        """
+        Saves metadata about processing. 
+        """
         files = glob.glob(pathlib.Path(self.output_directory, ".temp", "*.pkl").as_posix())
         for file in files:
             filename = pathlib.Path(file).name
@@ -595,22 +611,18 @@ class AutoPipeline(Pipeline):
                 # print("sadf123", metadata)
             np.warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning)
             self.output_df.loc[subject_id, list(metadata.keys())] = list(metadata.values()) #subject id targets the rows with that subject id and it is reassigning all the metadata values by key
-            # pd.set_option('display.max_rows', None)
-            # pd.set_option('display.max_columns', None)
-            # pd.set_option('display.width', None)
-            # print("asdfjlkasdjfkajfshg", self.output_df.head())
+            
         folder_renames = {}
         for col in self.output_df.columns:
             if col.startswith("folder"):
                 self.output_df[col] = self.output_df[col].apply(lambda x: x if not isinstance(x, str) else pathlib.Path(x).as_posix().split(self.input_directory)[1][1:]) # rel path, exclude the slash at the beginning
                 folder_renames[col] = f"input_{col}"
         self.output_df.rename(columns=folder_renames, inplace=True) #append input_ to the column name
-        # print("df in autopipe")
-        # print(self.output_df.iloc[0])
         self.output_df.to_csv(self.output_df_path) #dataset.csv
 
         shutil.rmtree(pathlib.Path(self.output_directory, ".temp").as_posix())
 
+        # Save dataset json
         if self.is_nnunet: #dataset.json for nnunet and .sh file to run to process it
             imagests_path = pathlib.Path(self.output_directory, "imagesTs").as_posix()
             images_test_location = imagests_path if os.path.exists(imagests_path) else None
@@ -638,6 +650,8 @@ class AutoPipeline(Pipeline):
                 output += 'done'
                 f.write(output)
             markdown_report_images(self.output_directory, self.total_modality_counter) #images saved to the output directory
+        
+        # Save summary info (factor into different file)
         markdown_path = pathlib.Path(self.output_directory, "report.md").as_posix()
         with open(markdown_path, "w", newline="\n") as f:
             output = "# Dataset Report\n\n"
@@ -648,6 +662,7 @@ class AutoPipeline(Pipeline):
                 formatted_list = "\n\t".join(self.broken_patients)
                 output += f"{formatted_list}\n"
                 output += "</details>\n\n"
+
             if self.is_nnunet:
                 output += "## Train Test Split\n\n"
                 # pie_path = pathlib.Path(self.output_directory, "markdown_images", "nnunet_train_test_pie.png").as_posix()
@@ -659,13 +674,11 @@ class AutoPipeline(Pipeline):
                 output += f"![Pie Chart of Image Modality Distribution]({bar_path})\n\n"
             f.write(output)
 
-
-
     def run(self):
         """Execute the pipeline, possibly in parallel.
         """
         # Joblib prints progress to stdout if verbose > 50
-        verbose = 51 if self.show_progress else 0
+        verbose = 51 if self.v or self.show_progress else 0
 
         subject_ids = self._get_loader_subject_ids()
         patient_ids = []
@@ -713,32 +726,21 @@ class AutoPipeline(Pipeline):
 
 def main():
     args = parser()
+    args_dict = vars(args)
+    
+    if args.continue_processing:
+        try:
+            with open(pathlib.Path(args.output_directory, ".temp", "init_parameters.pkl").as_posix(), "rb") as f:
+                args_dict = dill.load(f)
+        except:
+            print("Could not resume processing. Starting processing from the beginning.")
+
     print('initializing AutoPipeline...')
-    pipeline = AutoPipeline(args.input_directory,
-                            args.output_directory,
-                            modalities=args.modalities,
-                            visualize=args.visualize,
-                            spacing=args.spacing,
-                            n_jobs=args.n_jobs,
-                            show_progress=args.show_progress,
-                            warn_on_error=args.warn_on_error,
-                            overwrite=args.overwrite,
-                            is_nnunet=args.nnunet,
-                            train_size=args.train_size,
-                            random_state=args.random_state,
-                            read_yaml_label_names=args.read_yaml_label_names,
-                            ignore_missing_regex=args.ignore_missing_regex,
-                            roi_yaml_path=args.roi_yaml_path,
-                            custom_train_test_split=args.custom_train_test_split,
-                            is_nnunet_inference=args.is_nnunet_inference,
-                            dataset_json_path=args.dataset_json_path,
-                            continue_processing=args.continue_processing,
-                            dry_run=args.dry_run)
+    pipeline = AutoPipeline(**args_dict)
+    
     if not args.dry_run:
         print(f'starting AutoPipeline...')
         pipeline.run()
-
-
         print('finished AutoPipeline!')
     else:
         print('dry run complete, no processing done')
