@@ -46,16 +46,16 @@ def parser():
     parser.add_argument("--ignore_missing_regex", default=False, action="store_true",
                         help="Whether to ignore patients with no ROI regexes that match the given ones. Will throw an error on patients without matches if this is not set.")
 
-    parser.add_argument("--roi_yaml_path", type=str,
+    parser.add_argument("--roi_yaml_path", type=str, default="",
                         help="Path to the YAML file defining ROI regexes")
 
     parser.add_argument("--custom_train_test_split", default=False, action="store_true",
                         help="Whether to use a custom train-test-split, stored in custom_train_test_split.yaml in the input directory.")
 
-    parser.add_argument("--is_nnunet_inference", default=False, action="store_true",
+    parser.add_argument("--nnunet_inference", default=False, action="store_true",
                         help="Whether to generate data for nnUNet inference.")
     
-    parser.add_argument("--dataset_json_path", type=str,
+    parser.add_argument("--dataset_json_path", type=str, default="",
                         help="Path to the dataset.json file defining image modality indices for nnUNet inference.")
 
     parser.add_argument("--continue_processing", default=False, action="store_true",
@@ -64,6 +64,18 @@ def parser():
     parser.add_argument("--dry_run", default=False, action="store_true",
                         help="Make a dry run of the pipeline, only producing the edge table and dataset.csv.")
 
+    parser.add_argument("--verbose", default=False, action="store_true",
+                        help="Verbose output flag.")
+
+    parser.add_argument("--update", default=False, action="store_true",
+                        help="Update crawled index. In other words, process from scratch.")
+
+    parser.add_argument("--roi_select_first", default=False, action="store_true",
+                        help="Only select first matching regex/ROI name.")
+
+    parser.add_argument("--roi_separate", default=False, action="store_true",
+                        help="Process each matching regex/ROI into separate masks. Each matched mask will be saved as ROI_n. (n = index from list of regex/ROIs)")
+
     # parser.add_argument("--custom_train_test_split_path", type=str,
     #                     help="Path to the YAML file defining the custom train-test-split.")
 
